@@ -10,12 +10,13 @@ from code_loader.contract.datasetclasses import SectionCallableInterface, InputH
     GroundTruthHandler, MetadataHandler, DatasetIntegrationSetup, VisualizerHandler, PreprocessResponse, \
     PreprocessHandler, VisualizerCallableInterface, CustomLossHandler, CustomCallableInterface, PredictionTypeHandler, \
     MetadataSectionCallableInterface, UnlabeledDataPreprocessHandler, CustomLayerHandler, MetricHandler, \
-    CustomCallableInterfaceMultiArgs, ConfusionMatrixCallableInterfaceMultiArgs, \
-    CustomMultipleReturnCallableInterfaceMultiArgs, VisualizerCallableReturnType
-from code_loader.contract.enums import LeapDataType
+    CustomCallableInterfaceMultiArgs, ConfusionMatrixCallableInterfaceMultiArgs, VisualizerCallableReturnType, \
+    CustomMultipleReturnCallableInterfaceMultiArgs, DatasetBaseHandler
+from code_loader.contract.enums import LeapDataType, DataStateEnum, DataStateType
+from code_loader.contract.responsedataclasses import DatasetTestResultPayload
 from code_loader.contract.visualizer_classes import map_leap_data_type_to_visualizer_class
 from code_loader.metrics.default_metrics import metrics_names_to_functions
-from code_loader.utils import to_numpy_return_wrapper
+from code_loader.utils import to_numpy_return_wrapper, get_shape
 from code_loader.visualizers.default_visualizers import DefaultVisualizer, \
     default_graph_visualizer, \
     default_image_visualizer, default_horizontal_bar_visualizer, default_word_visualizer, \
@@ -65,7 +66,7 @@ class LeapBinder:
 
         if visualizer_type.value not in map_leap_data_type_to_visualizer_class:
             raise Exception(
-                f'The visualizer_type is invalid. current visualizer_type: {visualizer_type}, '
+                f'The visualizer_type is invalid. current visualizer_type: {visualizer_type}, '  # type: ignore[attr-defined]
                 f'should be one of : {", ".join([arg.__name__ for arg in VisualizerCallableReturnType.__args__])}')
 
         func_annotations = function.__annotations__
@@ -76,9 +77,9 @@ class LeapBinder:
                   f"https://docs.python.org/3/library/typing.html")
         else:
             return_type = func_annotations["return"]
-            if return_type not in VisualizerCallableReturnType.__args__:
+            if return_type not in VisualizerCallableReturnType.__args__:  # type: ignore[attr-defined]
                 raise Exception(
-                    f'The return type of function {function.__name__} is invalid. current return type: {return_type}, '
+                    f'The return type of function {function.__name__} is invalid. current return type: {return_type}, '  # type: ignore[attr-defined]
                     f'should be one of : {", ".join([arg.__name__ for arg in VisualizerCallableReturnType.__args__])}')
 
             expected_return_type = map_leap_data_type_to_visualizer_class[visualizer_type.value]
@@ -86,7 +87,7 @@ class LeapBinder:
                 raise Exception(
                     f'The return type of function {function.__name__} is invalid. current return type: {return_type}, '
                     f'should be {expected_return_type}')
-        
+
         self.setup_container.visualizers.append(
             VisualizerHandler(name, function, visualizer_type, arg_names, heatmap_visualizer))
         self._visualizer_names.append(name)
@@ -114,8 +115,8 @@ class LeapBinder:
     @typechecked
     def add_custom_metric(self,
                           function: Union[CustomCallableInterfaceMultiArgs,
-                                          CustomMultipleReturnCallableInterfaceMultiArgs,
-                                          ConfusionMatrixCallableInterfaceMultiArgs],
+                          CustomMultipleReturnCallableInterfaceMultiArgs,
+                          ConfusionMatrixCallableInterfaceMultiArgs],
                           name: str) -> None:
         arg_names = inspect.getfullargspec(function)[0]
         self.setup_container.metrics.append(MetricHandler(name, function, arg_names))
@@ -140,3 +141,87 @@ class LeapBinder:
         init_args = inspect.getfullargspec(custom_layer.__init__)[0][1:]
         call_args = inspect.getfullargspec(custom_layer.call)[0][1:]
         self.setup_container.custom_layers[name] = CustomLayerHandler(name, custom_layer, init_args, call_args)
+
+    def check_preprocess(self, preprocess_result: Dict[DataStateEnum, PreprocessResponse]) -> None:
+        preprocess_handler = self.setup_container.preprocess
+        unlabeled_preprocess_handler = self.setup_container.unlabeled_data_preprocess
+
+        if preprocess_handler is None:
+            raise Exception('None preprocess_handler')
+
+        for state, preprocess_response in preprocess_result.items():
+            if preprocess_response.length is None or preprocess_response.length <= 0:
+                raise Exception('Invalid dataset length')
+            if unlabeled_preprocess_handler is not None and state == DataStateEnum.unlabeled:
+                unlabeled_preprocess_handler.data_length = preprocess_response.length
+            else:
+                state_type = DataStateType(state.name)
+                preprocess_handler.data_length[state_type] = preprocess_response.length
+
+    def get_preprocess_result(self) -> Dict[DataStateEnum, PreprocessResponse]:
+        preprocess = self.setup_container.preprocess
+        if preprocess is None:
+            raise Exception("Please make sure you call the leap_binder.set_preprocess method")
+        preprocess_results = preprocess.function()
+        preprocess_result_dict = {
+            DataStateEnum(i): preprocess_result
+            for i, preprocess_result in enumerate(preprocess_results)
+        }
+
+        unlabeled_preprocess = self.setup_container.unlabeled_data_preprocess
+        if unlabeled_preprocess is not None:
+            preprocess_result_dict[DataStateEnum.unlabeled] = unlabeled_preprocess.function()
+
+        return preprocess_result_dict
+
+    def _get_all_dataset_base_handlers(self) -> List[Union[DatasetBaseHandler, MetadataHandler]]:
+        all_dataset_base_handlers: List[Union[DatasetBaseHandler, MetadataHandler]] = []
+        all_dataset_base_handlers.extend(self.setup_container.inputs)
+        all_dataset_base_handlers.extend(self.setup_container.ground_truths)
+        all_dataset_base_handlers.extend(self.setup_container.metadata)
+        return all_dataset_base_handlers
+
+    @staticmethod
+    def check_handler(
+            preprocess_response: PreprocessResponse, test_result: List[DatasetTestResultPayload],
+            dataset_base_handler: Union[DatasetBaseHandler, MetadataHandler]) -> List[DatasetTestResultPayload]:
+        raw_result = dataset_base_handler.function(0, preprocess_response)
+        handler_type = 'metadata' if isinstance(dataset_base_handler, MetadataHandler) else None
+        if isinstance(dataset_base_handler, MetadataHandler) and isinstance(raw_result, dict):
+            metadata_test_result_payloads = [
+                DatasetTestResultPayload(f'{dataset_base_handler.name}_{single_metadata_name}')
+                for single_metadata_name, single_metadata_result in raw_result.items()
+            ]
+            for i, (single_metadata_name, single_metadata_result) in enumerate(raw_result.items()):
+                metadata_test_result = metadata_test_result_payloads[i]
+                assert isinstance(single_metadata_result, (float, int, str, bool))
+                result_shape = get_shape(single_metadata_result)
+                metadata_test_result.shape = result_shape
+                metadata_test_result.raw_result = single_metadata_result
+                metadata_test_result.handler_type = handler_type
+            test_result = metadata_test_result_payloads
+        else:
+            assert not isinstance(raw_result, dict)
+            result_shape = get_shape(raw_result)
+            test_result[0].shape = result_shape
+            test_result[0].raw_result = raw_result
+            test_result[0].handler_type = handler_type
+
+            # setting shape in setup for all encoders
+            if isinstance(dataset_base_handler, (InputHandler, GroundTruthHandler)):
+                dataset_base_handler.shape = result_shape
+        return test_result
+
+    def check_handlers(self, preprocess_result: Dict[DataStateEnum, PreprocessResponse]) -> None:
+        dataset_base_handlers: List[Union[DatasetBaseHandler, MetadataHandler]] = self._get_all_dataset_base_handlers()
+        for dataset_base_handler in dataset_base_handlers:
+            test_result = [DatasetTestResultPayload(dataset_base_handler.name)]
+            for state, preprocess_response in preprocess_result.items():
+                if state == DataStateEnum.unlabeled and isinstance(dataset_base_handler, GroundTruthHandler):
+                    continue
+                self.check_handler(preprocess_response, test_result, dataset_base_handler)
+
+    def check(self) -> None:
+        preprocess_result = self.get_preprocess_result()
+        self.check_preprocess(preprocess_result)
+        self.check_handlers(preprocess_result)
